/* poller - front-end for waiting for I/O functions
   Copyright (C) 2010  Marek Behun

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#include <stdint.h>

#ifdef HAVE_EPOLL_WAIT

# include <sys/epoll.h>
# define POLLERIN EPOLLIN
# define POLLERPRI EPOLLPRI
# define POLLEROUT EPOLLOUT
# define POLLERHUP EPOLLHUP
# define POLLERERR EPOLLERR

typedef epoll_data_t pollerdata_t;
typedef uint32_t pollerev_t;

#else /* not HAVE_EPOLL_WAIT */

# include <poll.h>
# define POLLERIN POLLIN
# define POLLERPRI POLLPRI
# define POLLEROUT POLLOUT
# define POLLERHUP POLLHUP
# define POLLERERR POLLERR

union _pollerdata_union;
typedef union _pollerdata_union pollerdata_t;

union _pollerdata_union
{
  void * ptr;
  int fd;
  uint32_t u32;
  uint64_t u64;
};

typedef short pollerev_t;

#endif /* not HAVE_EPOLL_WAIT */

typedef void (*pollercb_t) (pollerdata_t data, pollerev_t events);

int poller_create ();
int poller_destroy (int inst);
int poller_add (int inst, int fd, pollerev_t events, pollerdata_t data);
int poller_modify (int inst, int fd, pollerev_t events, pollerdata_t data);
int poller_remove (int inst, int fd);
int poller_poll (int inst, int timeout, pollercb_t callback);
